@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -78,6 +78,8 @@ import com.openexchange.admin.rmi.exceptions.StorageException;
 
 public abstract class ListCore extends UserAbstraction {
 
+    private static final Object USER_ATTRIBUTES = "UserAttributes";
+
     protected final void setOptions(final AdminParser parser) {
         setDefaultCommandLineOptions(parser);
 
@@ -306,7 +308,11 @@ public abstract class ListCore extends UserAbstraction {
                 } else if (returntype.equals(JAVA_UTIL_HASH_SET)) {
                     datarow.add(hashtostring((HashSet<?>)methodandnames.getMethod().invoke(user, (Object[]) null)));
                 } else if (returntype.equals(JAVA_UTIL_MAP)) {
-                    datarow.add(maptostring((HashMap<?,?>)methodandnames.getMethod().invoke(user, (Object[]) null)));
+                    if(methodandnames.getName().equals(USER_ATTRIBUTES)) {
+                        datarow.add(userattributestostring((Map<String, Map<String, String>>)methodandnames.getMethod().invoke(user, (Object[]) null)));
+                    } else {
+                        datarow.add(maptostring((HashMap<?,?>)methodandnames.getMethod().invoke(user, (Object[]) null)));
+                    }
                 } else if (returntype.equals(JAVA_UTIL_TIME_ZONE)) {
                     datarow.add(timezonetostring((TimeZone)methodandnames.getMethod().invoke(user, (Object[]) null)));
                 } else if (returntype.equals(JAVA_UTIL_LOCALE)) {
@@ -354,6 +360,18 @@ public abstract class ListCore extends UserAbstraction {
         doCSVOutput(columnnames, data);
     }
 
+    private String userattributestostring(Map<String, Map<String, String>> dynamicAttributes) {
+        if(dynamicAttributes.size() == 0) {
+            return "";
+        }
+        StringBuilder builder = new StringBuilder(128);
+        for(Map.Entry<String, Map<String, String>> namespaced : dynamicAttributes.entrySet()) {
+            builder.append(namespaced.getKey()).append("=[").append(maptostring(namespaced.getValue())).append("],");
+        }
+        builder.deleteCharAt(builder.length()-1);
+        return builder.toString();
+    }
+
     protected abstract ArrayList<String> getDataOfAllExtensions(final User user) throws IllegalArgumentException, IllegalAccessException, InvocationTargetException;
 
     protected abstract ArrayList<String> getColumnsOfAllExtensions(final User user);
