@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,7 @@
 
 package com.openexchange.admin.storage.mysqlStorage;
 
+import static com.openexchange.tools.sql.DBUtils.autocommit;
 import static com.openexchange.tools.sql.DBUtils.closeSQLStuff;
 import static com.openexchange.tools.sql.DBUtils.rollback;
 import java.sql.Connection;
@@ -60,21 +61,20 @@ import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
-
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
-
 import com.openexchange.admin.daemons.ClientAdminThread;
 import com.openexchange.admin.exceptions.OXGenericException;
-import com.openexchange.admin.properties.AdminProperties;
 import com.openexchange.admin.rmi.dataobjects.Database;
 import com.openexchange.admin.rmi.exceptions.StorageException;
 import com.openexchange.admin.storage.sqlStorage.CreateTableRegistry;
 import com.openexchange.admin.tools.AdminCache;
 import com.openexchange.database.CreateTableService;
 import com.openexchange.groupware.AbstractOXException;
-import com.openexchange.groupware.update.UpdateTaskCollection;
-import com.openexchange.tools.sql.DBUtils;
+import com.openexchange.groupware.update.SchemaException;
+import com.openexchange.groupware.update.SchemaStore;
+import com.openexchange.groupware.update.UpdateTask;
+import com.openexchange.groupware.update.Updater;
 
 public class OXUtilMySQLStorageCommon {
 
@@ -113,15 +113,18 @@ public class OXUtilMySQLStorageCommon {
             con.setCatalog(db.getScheme());
             pumpData2DatabaseOld(con, createTableStatements);
             pumpData2DatabaseNew(con, CreateTableRegistry.getInstance().getList());
-            initVersionTable(con);
+            initUpdateTaskTable(con);
             con.commit();
         } catch (SQLException e) {
-            DBUtils.rollback(con);
+            rollback(con);
+            deleteDatabase(con, db);
             throw new StorageException(e.toString());
         } catch (StorageException e) {
-            DBUtils.rollback(con);
+            rollback(con);
+            deleteDatabase(con, db);
             throw e;
         } finally {
+            autocommit(con);
             cache.closeSimpleConnection(con);
         }
     }
@@ -226,20 +229,15 @@ public class OXUtilMySQLStorageCommon {
         return null;
     }
 
-    public static final void initVersionTable(Connection con) throws StorageException {
-        PreparedStatement stmt = null;
+    private void initUpdateTaskTable(Connection con) throws StorageException {
+        UpdateTask[] tasks = Updater.getInstance().getAvailableUpdateTasks();
+        SchemaStore store = SchemaStore.getInstance();
         try {
-            stmt = con.prepareStatement("INSERT INTO version (version,locked,gw_compatible,admin_compatible,server) VALUES(?,?,?,?,?);");
-            stmt.setInt(1, UpdateTaskCollection.getHighestVersion());
-            stmt.setInt(2, 0);
-            stmt.setInt(3, 1);
-            stmt.setInt(4, 1);
-            stmt.setString(5, cache.getProperties().getProp(AdminProperties.Prop.SERVER_NAME, "local"));
-            stmt.executeUpdate();
-        } catch (SQLException e) {
+            for (UpdateTask task : tasks) {
+                store.addExecutedTask(con, task.getClass().getName(), true);
+            }
+        } catch (SchemaException e) {
             throw new StorageException(e.getMessage(), e);
-        } finally {
-            DBUtils.closeSQLStuff(stmt);
         }
     }
 
@@ -254,6 +252,14 @@ public class OXUtilMySQLStorageCommon {
             LOG.error("Driver not found to create database ", e);
             throw new StorageException(e);
         }
+        try {
+            deleteDatabase(con, db);
+        } finally {
+            cache.closeSimpleConnection(con);
+        }
+    }
+
+    private void deleteDatabase(final Connection con, Database db) throws StorageException {
         Statement stmt = null;
         try {
             con.setAutoCommit(false);
@@ -261,12 +267,10 @@ public class OXUtilMySQLStorageCommon {
             stmt.executeUpdate("DROP DATABASE IF EXISTS `" + db.getScheme() + "`");
             con.commit();
         } catch (SQLException e) {
-            LOG.error("SQL Error", e);
             rollback(con);
             throw new StorageException(e.getMessage(), e);
         } finally {
             closeSQLStuff(stmt);
-            cache.closeSimpleConnection(con);
         }
     }
 
